#-----------------------------------------------------------------------------
# Superdot
# 12/25/03
# Copyright (c) 2003, David G. Drumright.  All rights reserved.
# Set the real Poser display to a camera and light setting.
# Left-click on one box to link that box to the current setting.
# Right-click on a box to re-apply its setting.
#-----------------------------------------------------------------------------
import os
import sys
from Tkinter import *
import poser
scene=poser.Scene()
if float(poser.Version()) < 5.0:
    import _TLmodule15
    ShowBMP = _TLmodule15.ShowBMP
else:
    import _TLmodule22
    ShowBMP = _TLmodule22.ShowBMP

BasePathUncut=poser.AppLocation()
BasePath=BasePathUncut[:(-len('poser.exe'))]
MyPath=os.path.normpath(BasePath+'Runtime/Python/Timeline/')

#-----------------------------------------------------------------------------
#-----------------------------------------------------------------------------
# Next section is TK
#------------------------------------------------------------

class App:
    def __init__(self, master, textMessage):
        self.master = master
        master.title("SuperDot")

        self.CamSettings={}
        self.LiteSettings={}
        self.OriginalCamSettings=[]
        self.OriginalLiteSettings=[]
        self.HoldCamSettings=[]
        self.HoldLiteSettings=[]

        self.ButtonFrame=Frame(self.master)
        self.ButtonFrame.grid(row=1,column=0)
        self.PictureFrame=Frame(self.master)
        self.PictureFrame.grid(row=2,column=0)

        self.buttonRefresh = Button(self.ButtonFrame, text="Update", command=self.FillFrame)
        self.buttonRefresh.grid(row=0, column=0)
        self.Instructions = Label(self.ButtonFrame, text="  Left-click on one box to hold current state;    Right-click on one box to restore that state.  ").grid(row=0,column=1)
        self.buttonRestore = Button(self.ButtonFrame, text="Original", command=self.RestoreOriginals)
        self.buttonRestore.grid(row=0, column=2)

        self.OneLabel=[]
        self.OneFrame=[]
        for i in range(7):
            L=Entry(self.PictureFrame,text=str(i+1))
            L.grid(row=0,column=i)
            self.OneLabel.append(L)
            F = Canvas(self.PictureFrame,width=100,height=100,bg="white",borderwidth=2,relief=RIDGE)
            F.grid(row=1,column=i)
            if   i==0:
                F.bind("<Button-1>",self.F0Get)
                F.bind("<Button-3>",self.F0Put)
            elif i==1:
                F.bind("<Button-1>",self.F1Get)
                F.bind("<Button-3>",self.F1Put)
            elif i==2:
                F.bind("<Button-1>",self.F2Get)
                F.bind("<Button-3>",self.F2Put)
            elif i==3:
                F.bind("<Button-1>",self.F3Get)
                F.bind("<Button-3>",self.F3Put)
            elif i==4:
                F.bind("<Button-1>",self.F4Get)
                F.bind("<Button-3>",self.F4Put)
            elif i==5:
                F.bind("<Button-1>",self.F5Get)
                F.bind("<Button-3>",self.F5Put)
            elif i==6:
                F.bind("<Button-1>",self.F6Get)
                F.bind("<Button-3>",self.F6Put)

            self.OneFrame.append(F)

        # Exit cleanly
        self.master.protocol('WM_DELETE_WINDOW', self.HandleCancel)

        # Begin by storing default settings
        self.HoldOriginals()


    # - - - - - - - - - - - - - - - - - -
    # - - - - - - - - - - - - - - - - - -

    def Update(self):
        scene.ProcessSomeEvents(1)
        root.lift()
        root.after(50, self.Update)

    # - - - - - - - - - - - - - - - - - -

    def HoldDefaults(self):
        self.HoldCamSettings  = self.GetCam()
        self.HoldLiteSettings = self.GetLites()

    # - - - - - - - - - - - - - - - - - -
    def HoldOriginals(self):
        self.OriginalCamSettings  = self.GetCam()
        self.OriginalLiteSettings = self.GetLites()
    # - - - - - - - - - - - - - - - - - -

    def RestoreDefaults(self):
        self.PutCam(self.HoldCamSettings)
        self.PutLites(self.HoldLiteSettings)

    # - - - - - - - - - - - - - - - - - -
    def RestoreOriginals(self):
        self.PutCam(self.OriginalCamSettings)
        self.PutLites(self.OriginalLiteSettings)

    # - - - - - - - - - - - - - - - - - -
    # - - - - - - - - - - - - - - - - - -
    def F0Get(self,event):  # Left-click in frame 0
        self.LiteSettings[0]=self.GetLites()
        self.CamSettings[0]=self.GetCam()
        self.FillFrame()
        scene.DrawAll()
    # - - - - - - - - - - - - - - - - - -
    def F1Get(self,event):  # Left-click in frame 1
        self.LiteSettings[1]=self.GetLites()
        self.CamSettings[1]=self.GetCam()
        self.FillFrame()
        scene.DrawAll()
    # - - - - - - - - - - - - - - - - - -
    def F2Get(self,event):  # Left-click in frame 2
        self.LiteSettings[2]=self.GetLites()
        self.CamSettings[2]=self.GetCam()
        self.FillFrame()
        scene.DrawAll()
    # - - - - - - - - - - - - - - - - - -
    def F3Get(self,event):  # Left-click in frame 3
        self.LiteSettings[3]=self.GetLites()
        self.CamSettings[3]=self.GetCam()
        self.FillFrame()
        scene.DrawAll()
    # - - - - - - - - - - - - - - - - - -
    def F4Get(self,event):  # Left-click in frame 4
        self.LiteSettings[4]=self.GetLites()
        self.CamSettings[4]=self.GetCam()
        self.FillFrame()
        scene.DrawAll()
    # - - - - - - - - - - - - - - - - - -
    def F5Get(self,event):  # Left-click in frame 5
        self.LiteSettings[5]=self.GetLites()
        self.CamSettings[5]=self.GetCam()
        self.FillFrame()
        scene.DrawAll()
    # - - - - - - - - - - - - - - - - - -
    def F6Get(self,event):  # Left-click in frame 6
        self.LiteSettings[6]=self.GetLites()
        self.CamSettings[6]=self.GetCam()
        self.FillFrame()
        scene.DrawAll()
    # - - - - - - - - - - - - - - - - - -

    # - - - - - - - - - - - - - - - - - -
    def F0Put(self,event):  # Right-click in frame 0
        self.PutLites(self.LiteSettings[0])
        self.PutCam(self.CamSettings[0])
        scene.DrawAll()
    # - - - - - - - - - - - - - - - - - -
    def F1Put(self,event):  # Right-click in frame 1
        self.PutLites(self.LiteSettings[1])
        self.PutCam(self.CamSettings[1])
        scene.DrawAll()
    # - - - - - - - - - - - - - - - - - -
    def F2Put(self,event):  # Right-click in frame 2
        self.PutLites(self.LiteSettings[2])
        self.PutCam(self.CamSettings[2])
        scene.DrawAll()
    # - - - - - - - - - - - - - - - - - -
    def F3Put(self,event):  # Right-click in frame 3
        self.PutLites(self.LiteSettings[3])
        self.PutCam(self.CamSettings[3])
        scene.DrawAll()
    # - - - - - - - - - - - - - - - - - -
    def F4Put(self,event):  # Right-click in frame 4
        self.PutLites(self.LiteSettings[4])
        self.PutCam(self.CamSettings[4])
        scene.DrawAll()
    # - - - - - - - - - - - - - - - - - -
    def F5Put(self,event):  # Right-click in frame 5
        self.PutLites(self.LiteSettings[5])
        self.PutCam(self.CamSettings[5])
        scene.DrawAll()
    # - - - - - - - - - - - - - - - - - -
    def F6Put(self,event):  # Right-click in frame 6
        self.PutLites(self.LiteSettings[6])
        self.PutCam(self.CamSettings[6])
        scene.DrawAll()
    # - - - - - - - - - - - - - - - - - -
    # - - - - - - - - - - - - - - - - - -

    def PutLites(self,Settings):
        if not Settings: return # Probably should give more, but see if this helps
        for (OneLiteName,OneParmList) in Settings:
            TheLight = scene.Actor(OneLiteName)
            for (PN,PV) in OneParmList:
                TheLight.Parameter(PN).SetValue(PV)


    # - - - - - - - - - - - - - - - - - -
    def GetLites(self):
        # Build the list from current scene.
        Settings = []
        for OneLite in scene.Lights():
            OneLiteName = OneLite.Name()
            OneParmList = []
            for OneParm in OneLite.Parameters():
                PN = OneParm.Name()
                PV = OneParm.Value()
                OneParmList.append((PN,PV))
            Settings.append((OneLiteName,OneParmList))
        return Settings

    # - - - - - - - - - - - - - - - - - -

    def GetCam(self):
        # Build the list from current scene.
        OneCam = scene.CurrentCamera()
        OneCamName = OneCam.Name()
        OneParmList = []
        for OneParm in OneCam.Parameters():
            PN = OneParm.Name()
            PV = OneParm.Value()
            OneParmList.append((PN,PV))
        return (OneCamName,OneParmList)

    # - - - - - - - - - - - - - - - - - -

    def PutCam(self,Settings):
        # Only one cam, so simpler than lites
        try:
            (OneCamName,OneParmList) = Settings
        except:
            return
        TheCam = scene.Actor(OneCamName)
        for (PN,PV) in OneParmList:
            TheCam.Parameter(PN).SetValue(PV)
        scene.SetCurrentCamera(TheCam)

    # - - - - - - - - - - - - - - - - - -

    def FillFrame(self):
        self.HoldDefaults() # Keep whatever is here
        if not len(self.CamSettings.keys()): return
        for i in self.CamSettings.keys(): # Order doesn't matter here
            self.PutLites(self.LiteSettings[i])
            self.PutCam(self.CamSettings[i])
            scene.DrawAll()
            # Make name and save image
            TryName = os.path.abspath('%s/%d.BMP' % (MyPath,i))
            scene.SaveImage("bmp",TryName)
            # Find hwnd of this box, do the Show action
            Hwnd = self.OneFrame[i].winfo_id()
            retval = ShowBMP(Hwnd, TryName,4,4,100,100)
            if retval < 1: raise "Render error %d" % retval
        self.RestoreDefaults() # Restore whatever was here
    # - - - - - - - - - - - - - - - - - -

    def HandleCancel(self):
        self.RestoreOriginals()
        self.master.destroy()

#------------------------------------------------------------
# Activate the loop
#------------------------------------------------------------

root = Tk()
app = App(root, '')
app.Update()
root.mainloop()

#------------------------------------------------------------
# End TK loop.
#-----------------------------------------------------------------------------
